@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2022 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2024 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file in the Gmsh root directory for license information.
 // Please report all issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -621,10 +621,10 @@ static bool _tryToCollapseThatVertex2(GFace *gf, std::vector<MElement *> &e1,
   double worst_quality_old = 1.0;
   double worst_quality_new = 1.0;
 
-  int count = 0;
+  //int count = 0;
   for(std::size_t j = 0; j < e.size(); ++j) {
     if(e[j] != q) {
-      count++;
+      //count++;
       worst_quality_old = std::min(worst_quality_old, e[j]->etaShapeMeasure());
       v1->x() = pp.x();
       v1->y() = pp.y();
@@ -686,10 +686,10 @@ static bool _tryToCollapseThatVertex(GFace *gf, std::vector<MElement *> &e1,
   GPoint pp = gf->point(0.5 * (uu1 + uu2), 0.5 * (vv1 + vv2));
   double worst_quality_old = 1.0;
   double worst_quality_new = 1.0;
-  int count = 0;
+  //int count = 0;
   for(std::size_t j = 0; j < e.size(); ++j) {
     if(e[j] != q) {
-      count++;
+      //count++;
       worst_quality_old = std::min(worst_quality_old, e[j]->etaShapeMeasure());
       v1->x() = pp.x();
       v1->y() = pp.y();
@@ -771,6 +771,17 @@ static bool _isItAGoodIdeaToMoveThatVertex(GFace *gf,
   return true;
 }
 
+void getAllBoundaryLayerVertices(GFace *gf, std::set<MVertex *> &vs)
+{
+  vs.clear();
+  BoundaryLayerColumns *_columns = gf->getColumns();
+  if(!_columns) return;
+  for(auto it = _columns->_data.begin(); it != _columns->_data.end(); it++) {
+    BoundaryLayerData &data = it->second;
+    for(size_t i = 0; i < data._column.size(); i++) vs.insert(data._column[i]);
+  }
+}
+
 static bool has_none_of(std::set<MVertex *> const &touched, MVertex *const v1,
                         MVertex *const v2, MVertex *const v3, MVertex *const v4)
 {
@@ -810,6 +821,10 @@ static int _removeDiamonds(GFace *const gf)
     touched.insert(gf->triangles[i]->getVertex(2));
   }
 
+  std::set<MVertex *> vs;
+  getAllBoundaryLayerVertices(gf, vs);
+  touched.insert(vs.begin(), vs.end());
+
   for(std::size_t i = 0; i < gf->quadrangles.size(); i++) {
     MQuadrangle *const q = gf->quadrangles[i];
 
@@ -964,17 +979,6 @@ static void _relocate(GFace *gf, MVertex *ver,
   }
 }
 
-void getAllBoundaryLayerVertices(GFace *gf, std::set<MVertex *> &vs)
-{
-  vs.clear();
-  BoundaryLayerColumns *_columns = gf->getColumns();
-  if(!_columns) return;
-  for(auto it = _columns->_data.begin(); it != _columns->_data.end(); it++) {
-    BoundaryLayerData &data = it->second;
-    for(size_t i = 0; i < data._column.size(); i++) vs.insert(data._column[i]);
-  }
-}
-
 void laplaceSmoothing(GFace *gf, int niter, bool infinity_norm)
 {
   if((gf->triangles.size() > 0 && gf->triangles[0]->getPolynomialOrder() > 1) ||
@@ -1463,8 +1467,9 @@ void splitElementsInBoundaryLayerIfNeeded(GFace *gf)
     int n = fields->getNumBoundaryLayerFields();
     for(int i = 0; i < n; ++i) {
       Field *bl_field = fields->get(fields->getBoundaryLayerField(i));
-      if(bl_field == nullptr) continue;
+      if(!bl_field) continue;
       BoundaryLayerField *blf = dynamic_cast<BoundaryLayerField *>(bl_field);
+      if(!blf) continue;
       if(blf->iRecombine)
         ++numNoSplit;
       else
